#!/bin/ksh -p
# $Id: functions.sh,v 1.1.4.5.2.7.2.3 2016/04/08 17:57:55 rosacc Exp $

function dev_vert_copy
{
    typeset src="${1:?}"
    if [ ! -f "$src" ] ; then
        src=$(echo "$src" | sed 's/dev\/vert-/day\/vert-/')
        if [ ! -f "$src" ] ; then
            echo "no jar at dev or day: $src"
            exit 1
        fi
    fi    
    cp "$src" "${2:?}"
}

function read_properties_file
{
    file_readable "$1"

    typeset X KEY VAL

    cat "$1" | while read X
    do
    
        X=`echo $X | sed -e 's/^[ \t]*//' -e '/^[#!]/d' -e 's/\r//' `
    
        if [ -z "$X" ] ; then
    	    continue
        fi
    
        KEY=`echo $X | sed -n 's/^[ \t]*\([A-Za-z][A-Za-z_0-9]*\)[ \t]*[:=]\{0,1\}[ \t]*\(.*\)$/\1/p' `
    
        if [ -z "$KEY" ]; then
    	    continue
        fi
    
        VAL=`echo $X | sed -n 's/^[ \t]*\([A-Za-z][A-Za-z_0-9]*\)[ \t]*[:=]\{0,1\}[ \t]*\(.*\)$/\2/p' `
    
        eval ${KEY}=\"${VAL}\"
    
    done
}

function file_exists
{
    if [ ! -f "$1" ] ; then
        echo "file not found: \"$1\""
        exit 1
    fi
}

function file_readable
{
    file_exists "$1" "$2"
    
    if [ ! -r "$1" ] ; then
        echo "cannot read: $2\"$1\""
        exit 1
    fi
}   

function file_executable
{
    file_exists "$1" "$2"
    
    if [ ! -x "$1" ] ; then
        echo "cannot execute: $2\"$1\""
        exit 1
    fi
}    

function dir_exists
{
    if [ ! -d "$1" ] ; then
        echo "directory not found: $2\"$1\""
        exit 1
    fi
}

function writeable
{
    if [ ! -w "$1" ] ; then
        echo "cannot write: $2\"$1\""
        exit 1
    fi
}

function dir_writeable
{
    dir_exists "$1" "$2"
    writeable  "$1" "$2"
}

function check_file
{
    file_readable "${1:?}" "$2"
    echo "${1:?}"
}

function check_dir
{
    dir_exists "${1:?}" "$2"
    echo "${1:?}"
}

function init_common_env
{
    PROGRAM_DIR="$(absolute_path ${PROGRAM_NAME:?})"
    PROGRAM_NAME="${PROGRAM_NAME##*(?)/}"
    
    : ${SERVER_HOME:="$(absolute_path ${PROGRAM_DIR:?}/..)"}
    
    dir_writeable "${CONFIG_DIR:="${SERVER_HOME:?}/etc"}"       "CONFIG_DIR="
    dir_writeable "${LOGS_DIR:="${SERVER_HOME:?}/logs"}"        "LOGS_DIR="
    dir_writeable "${ARCHIVE_DIR:="${SERVER_HOME:?}/archive"}"  "ARCHIVE_DIR="
    dir_writeable "${TMPDIR:="${LOGS_DIR:?}"}"                    "TMPDIR="

    if [[ -z "${LIB_DIR}" && -d "${SERVER_HOME:?}/lib" ]] ; then
        LIB_DIR="${SERVER_HOME:?}/lib"
    fi
    
    file_executable "${JAVA:="${JAVA_HOME:?}/bin/java"}"        "JAVA="

    : ${ARCHIVE_COUNT_LIMIT:="30"}
    
    : ${JAVA_HEAP:="4G"}
    : ${JAVA_YOUNG_HEAP:=512M}
    
    : ${DATE_TIME:=$(date +\"%y%b%d_%H%M%S%Z\")}

    : ${ASCII_ENCODING:="ISO8859_1"}
    : ${EBCDIC_ENCODING:="cp1047"}
    : ${OSE_SYSTEM_ENCODING:="ascii"}
    
    : ${YEARCUTOFF:="1926"}
    
    TMP="${TMPDIR:?}/.${PROGRAM_NAME:?}.tmp.$$"
    echo >${TMP:?}
    trap "rm -f ${TMP:?}" 0 1 2 3 15

    if [ -n "${LIB_DIR}" ] ; then
        CLASS_DIRS="${CLASS_DIRS}:${LIB_DIR}"
    fi
    
    CLASSPATH="$CLASSPATH:$(check_dir ${CONFIG_DIR:?} CONFIG_DIR=)"
    
    init_os_env
    init_jvm_env
    
    OPTS="$OPTS $(assertion_opts)"
    OPTS="$OPTS $(encoding_opts)"
}

function init_os_env
{
    RSYNC="cp -fpr"

    if [ $(uname -s) = "AIX" ] ; then

        PSTREE="ps -T"
        ECHO_N="echo"
        IPCS_SED_XPRN='s/^[sm] \{1,\}\([A-Za-z0-9_]\{1,\}\).*/\1/'
    
    else
    
        PSTREE="pstree"
        ECHO_N="echo -e"
        IPCS_SED_XPRN='s/^ *\([A-Za-z0-9_]\{1,\}\).*/\1/'
    
    fi
}

function init_jvm_env
{
    "$JAVA" -version >"$TMP" 2>&1

    if grep -iq "AIX" "$TMP" ; then
    
        : ${PROVIDER_JAR:="${JAVA_HOME:?}/lib/ext/ibmjceprovider.jar"}

		# XXX gencon tests better that optavgpause
        OPTS="$OPTS -Xgcpolicy:gencon"
#        OPTS="$OPTS -Xgcpolicy:optavgpause"

    else 
        
        : ${PROVIDER_JAR:="${JAVA_HOME:?}/lib/ext/sunjce_provider.jar"}
        
        # XXX using concurrent new and old generation GC was good for a large heap
        OPTS="$OPTS -XX:+UseConcMarkSweepGC -XX:+UseParNewGC"
#        OPTS="$OPTS -XX:+UseParallelGC"
        OPTS="$OPTS -XX:+AggressiveOpts"
        
    fi
}

function init_db_env
{
    case "${MEH_JDBC_URL:?}" in
    
    *:db2:*)
    
        CLASSPATH="$CLASSPATH:$(check_file ${DB_HOME:?}/java/db2jcc4.jar)"
        CLASSPATH="$CLASSPATH:$(check_file ${DB_HOME:?}/java/db2jcc_license_cu.jar)"
        ;;
    
    *:oracle:*)
    
        CLASSPATH="$CLASSPATH:$(check_file ${DB_HOME:?}/jdbc/lib/ojdbc7.jar)"
        LIBRARY_PATH="$LIBRARY_PATH:$(check_dir ${DB_HOME:?}/lib)"
        ;;
    
    "")

        ;;
    
    *)
    
        echo "ERROR: MEH_JDBC_URL=$MEH_JDBC_URL"
        exit 1
        ;;
    
    esac
}

function init_mq_env
{
    if [[ -n "$MQ_HOME" ]]; then
      CLASS_DIRS="$CLASS_DIRS:$(check_dir ${MQ_HOME:?}/java/lib)"
      LIBRARY_PATH="$LIBRARY_PATH:$(check_dir ${MQ_HOME:?}/java/lib64)"
    else
      if [[ $JMS_PROVIDER == "MqSeries" ]]; then
        echo "MQ_HOME not set, but you have configured it as your JMS provider.  MQ_HOME needs to be set."
        exit 1
      fi
    fi


}

function init_server_env
{
    init_common_env
    
    : ${LOG4J_CONFIG:="log4j-$(echo ${MODE:?} | tr 'A-Z' 'a-z').xml"}
    
    : ${OSE_XML:="file://${CONFIG_DIR:?}/ose.xml"}
    : ${OSE_BEAN:="ose"}
    : ${OSE_CLASS:="com.sas.finance.fraud.engine.OSE"}        

    init_running_process_env

    init_mq_env
    init_db_env

    # Java Cryptography Extension (JCE)
    CLASSPATH="$CLASSPATH:$(check_file ${PROVIDER_JAR:?})"

    if [ -n "${LIB_DIR}" ] ; then
        LIBRARY_PATH="$LIBRARY_PATH:${LIB_DIR:?}"
    fi
    
    LIBRARY_PATH="${LIBRARY_PATH}:"`which sas`'exe';

    # OMR SSL certificate exchange, the /. is intentional
    OPTS="$OPTS -Djava.security.egd=file:/dev/./urandom"
    
    OPTS="$OPTS -Dsas.yearcutoff=${YEARCUTOFF:?}"
    OPTS="$OPTS -Djava.library.path=$LIBRARY_PATH"

    OPTS="$OPTS $(vjr_opts)"
}

function absolute_path
{
    typeset X="$1"
    if [ -f "$X" ] ; then
       X="$(dirname $X)"
    fi
    echo "$(cd $X ; pwd -P)"
}

function assertion_opts
{
    if [ "${MODE:?}" != "PROD" ]; then
        echo "-ea -esa"
    fi
}

function encoding_opts
{
    typeset X="-Dascii.encoding=$ASCII_ENCODING"
    X="$X -Debcdic.encoding=$EBCDIC_ENCODING"
    X="$X -Dtransaction.apiEncoding=$OSE_SYSTEM_ENCODING"
    echo $X
}

function clear_env
{
    MQ_HOME=""
    JAVA_HOME=""
    CLASSPATH=""
    MODE="TEST"
}

function debug_opts
{
    echo "-agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address=${DEBUG_PORT:="7777"}"
}

function vjr_opts
{
    : ${PICKLIST:="${CONFIG_DIR}/picklist"}

    typeset X="-Djava.system.class.loader=com.sas.app.AppClassLoader"

    X="$X -Dsas.app.launch.config=$(check_file ${PICKLIST:?} PICKLIST=)"

    X="$X -classpath $(check_file ${VJR_HOME:?}/eclipse/plugins/sas.launcher.jar)"
    X="$X -Dsas.app.repository.path=$(check_dir ${VJR_HOME:?}/eclipse)"

    X="$X -Dsas.app.class.dirs=$CLASS_DIRS"
    X="$X -Dsas.app.class.path=$CLASSPATH"

    echo "$X"
}

function init_running_process_env
{
    JPS_OSE=$(ps -e -o pid= -o pgid= -o args= | grep "${JAVA:?}" | grep "${OSE_CLASS:?} -config ${OSE_XML:?} ${OSE_BEAN:?}" || :)
    OSE_PID=$( echo "$JPS_OSE" | sed -n 's/^[ \t]*\([0-9]\{1,\}\)[ \t]\{1,\}\([0-9]\{1,\}\)[ \t]\{1,\}\(.*\)/\1/p')
    OSE_PGID=$(echo "$JPS_OSE" | sed -n 's/^[ \t]*\([0-9]\{1,\}\)[ \t]\{1,\}\([0-9]\{1,\}\)[ \t]\{1,\}\(.*\)/\2/p')
    OSE_ARGS=$(echo "$JPS_OSE" | sed -n 's/^[ \t]*\([0-9]\{1,\}\)[ \t]\{1,\}\([0-9]\{1,\}\)[ \t]\{1,\}\(.*\)/\3/p')
}

function unknown_command
{
    echo "unknown command: '$1'"
    usage
    exit 1
}
